from parsimonious.grammar import Grammar

# This grammar is based off of the Modelica 3.3 standard.
# https://www.modelica.org/documents/ModelicaSpec33.pdf

modelica_parser = Grammar(r"""
    #===============================================================
    # STORED DEFINITION
    #===============================================================
    stored_definition = _ (within name? semicolon)?
        (_ final? class_definition semicolon)

    #===============================================================
    # CLASS DEFINITION
    #===============================================================
    class_definition = encapsulated? class_prefixes class_specifier

    class_prefixes = partial?
        (class/model/(operator? record)/block/
        (expandable? connector)/type/package/
        ((pure/impure)? operator? function)/
        operator)

    class_specifier = long_class_specifier/
        short_class_specifier/
        der_class_specifier/
        extends_class_specifier

    long_class_specifier = ident string_comment
        composition end ident

    short_class_specifier = (ident equals base_prefix name
        array_subscripts? class_modification? comment) /
        (ident  equals enumeration lparen (enum_list/colon) rparen
        comment)

    der_class_specifier = ident equals der lparen name
        (comma ident)+ rparen comment

    extends_class_specifier = extends ident class_modification?
        string_comment composition end ident

    base_prefix = type_prefix

    enum_list = enumeration_literal (comma enumeration_literal)*

    enumeration_literal = ident comment

    composition = element_list (((public/protected)
       element_list)/ equation_section/ algorithm_section)*
       (external language_specification?
       external_function_call? annotation? semicolon)?
       (annotation semicolon)?

    language_specification = string

    external_function_call = (component_reference equals)? ident
        lparen expression_list? rparen

    # notice we do a lookahead assertion here that is PEG but not EBNF
    # to ensure that end is not consumed as an ident
    element_list = (!(end/equation/algorithm)
        ((element semicolon)/(annotation semicolon)))*

    element = import_clause /  extends_clause
         / (redeclare? final? inner? outer? (
         (class_definition / component_clause)
         /(replaceable (class_definition / component_clause)
         constraining_clause comment)))

    import_clause = import ( (ident equals name) /
        (name (period (times / (lbrace import_list rbrace)) )?)) comment

    import_list = ident (comma import_list)?

    #===============================================================
    # EXTENDS
    #===============================================================
    extends_clause = extends name class_modification? annotation?

    constraining_clause = constrainedby name class_modification?

    #===============================================================
    # COMPONENT CLAUSE
    #===============================================================
    component_clause = type_prefix type_specifier array_subscripts?
       component_list

    type_prefix = (flow/ stream)? (discrete/parameter/constant)?
    (input/output)?

    type_specifier = name

    component_list = component_declaration (comma
      component_declaration)*

    component_declaration = declaration condition_attribute? comment?

    condition_attribute = if expression

    declaration = ident array_subscripts? modification?

    #===============================================================
    # MODIFICATION
    #===============================================================
    modification = (class modification ( equals expression)?) /
        ( equals expression) / (assign expression)

    class_modification = lparen argument_list? rparen

    argument_list = argument (comma argument)*

    argument = element_modification_or_replaceable /
        element_redeclaration

    element_modification_or_replaceable =
        each? final? (element_modification / element_replaceable)

    element_modification = name modification? string_comment

    element_redeclaration = redeclare each? final?

    element_replaceable = replaceable (short_class_definition /
        component_clause1) constraining_clause?

    component_clause1 = type_prefix type_specifier
        component_declaration1

    component_declaration1 = declaration comment

    short_class_definition = class_prefixes ident equals ((
        base_prefix name array_subscripts? class_modification?
        comment) / (enumeration lparen ( enum_list? / colon )
        rparen comment ))


    #===============================================================
    # EQUATION
    #===============================================================
    equation_section = initial? equation (equation_expr semicolon)*

    algorithm_section = initial? algorithm (statement semicolon)*

    # note there is also an equation keywords so we call the
    # expression equation_expr
    equation_expr = ((simple_expression equals expression)
        / if_equation / for_equation
        / connect_clause / when_equation
        / (name function_call_args)) comment

    statement = ((component_reference ( (assign expression)
        / function_call_args )) / ( lparen output_expression_list
        rparen assign component_reference function_call_args)
        / break / return / if_statement
        / for_statement / while_statement / when_statement )

    if_equation = if expression then
            (equation_expr semicolon)*
        (elseif expression then
            (equation_expr semicolon)* )*
        (else
            (equation_expr semicolon)* )?
        end if

    if_statement = if expression then
            (statement semicolon)*
        (elseif expression then
            (statement semicolon)*
        )*
        (else
            (statement semicolon)*
        )?
        end if

    for_equation = for for_indices loop
        (equation_expr semicolon)*
        end for

    for_statement = for for_indices loop
        (statement semicolon)*
        end for

    for_indices = for_index (comma for_index)*

    for_index = ident (in expression)?

    while_statement = while expression loop
        (statement semicolon)*
        end while

    when_equation = when expression then
            (equation semicolon)*
        (elsewhen expression then
            (equation semicolon)*
        )*
        end when

    when_statement = when expression then
            (statement semicolon)*
        (elsewhen expression then
            (statement semicolon)*
        )*
        end when

    connect_clause = connect lparen component_reference comma
        component_reference rparen

    #===============================================================
    # EXPRESSION
    #===============================================================
    expression = simple_expression /
        (if expression then expression
        (elseif expression then expression)* else expression)

    simple_expression = logical_expression
        (semicolon logical_expression
        (semicolon logical_expression)?)?

    logical_expression = logical_term (or logical_term)*

    logical_term = logical_factor (and logical_factor)*

    logical_factor = not? relation

    relation = arithmetic_expression (rel_op arithmetic_expression)?

    rel_op = less_than / less_than_or_equal / greater_than
        / greater_than_or_equal / equality / inequality

    arithmetic_expression = add_op? term (add_op term)*

    add_op = plus / minus / dot_plus / dot_minus

    term = factor (mul_op factor)*

    mul_op = times / divide / dot_times / dot_divide

    factor = primary ( (exp / dot_exp) primary)?

    primary = unsigned_number / string / false / true
        / ((name / der / initial) function_call_args)
        / component_reference
        / (lparen output_expression_list rparen)
        / (lbracket expression_list
            ( semicolon expression_list )* rbracket)
        / (lbrace function_arguments rbrace)
        / end

    name = period ? ident (period ident)*

    component_reference = (period ident array_subscripts?)+

    function_call_args = lparen function_arguments? rparen

    function_arguments = function argument
        ((comma function_arguments) / (for for_indices) /
        named_arguments)

    named_arguments = named_argument (comma named_arguments)?

    named_argument = ident equals function_argument

    function_argument = function name
        ((lparen named_arguments? rparen) / expression)

    output_expression_list = expression? (comma expression?)*

    expression_list = expression (comma expression)*

    array_subscripts = lbracket subscript (comma subscript)* rbracket

    subscript = colon / expression

    comment = string_comment annotation?

    string_comment = string ( plus string)*

    annotation = annotation class_modification

    #===============================================================
    # BASIC
    #===============================================================
    _ = ~'\s*'
    equals = '='_
    assign = ':='_
    semicolon = ';'_
    lparen = '('_
    rparen = ')'_
    lbracket = '{'_
    rbracket = '}'_
    colon = ':'_
    comma = ','_
    double_quote = '"'_
    single_quote = "'"_
    lbrace = '{'_
    rbrace = '}'_
    period = '.'_
    plus = '+'_
    dot_plus = '.+'_
    minus = '-'_
    dot_minus = '.-'_
    times = '*'_
    dot_times = '.*'_
    divide = '/'_
    dot_divide = './'_
    exp = '^'_
    dot_exp = '.^'_
    less_than = '<'_
    less_than_or_equal = '<='_
    greater_than = '>'_
    greater_than_or_equal = '>='_
    equality = '=='_
    inequality = '<>'_
    ident = (nondigit ( digit / nondigit )*_) / q_ident
    q_ident = single_quote (q_char / s_escape)+ single_quote
    string = double_quote (s_char/s_escape)* double_quote
    nondigit = ~'[_a-zA-Z]'
    s_char = ~r'[^"\\]*'u
    q_char = (nondigit/digit/~r'[#$%&()*+,-./:;<>=?@[]^\{}|~ ')
    s_escape = ~r'[\'"\?\\\a\b\f\n\r\t\v]'
    digit = ~'[0-9]'
    unsigned_integer = digit+
    unsigned_number = unsigned_integer ( '.' unsigned_integer?)?
        (('e'/'E') ('+'/'-')? unsigned_integer)?

    #===============================================================
    # KEYWORDS
    #===============================================================
    algorithm = 'algorithm'_
    and = 'and'_
    annotation = 'annotation'_
    assert = 'assert'_
    block = 'block'_
    break = 'break'_
    class = 'class'_
    connect = 'connect'_
    connector = 'connector'_
    constant = 'constant'_
    constrainedby = 'constrainedby'_
    der = 'der'_
    discrete = 'discrete'_
    each = 'each'_
    else = 'else'_
    elseif = 'elseif'_
    elsewhen = 'elsewhen'_
    encapsulated = 'encapsulated'_
    end = 'end'_
    enumeration = 'enumeration'_
    equation = 'equation'_
    expandable = 'expandable'_
    extends = 'extends'_
    external = 'external'_
    false = 'false'_
    final = 'final'_
    flow = 'flow'_
    for= 'for'_
    function = 'function'_
    if = 'if'_
    import = 'import'_
    impure = 'impure'_
    in = 'in'_
    initial = 'initial'_
    inner = 'inner'_
    input = 'input'_
    initial = 'initial'_
    loop = 'loop'_
    model = 'model'_
    not = 'not'_
    operator = 'operator'_
    or = 'or'_
    outer = 'outer'_
    output = 'output'_
    package = 'package'_
    parameter = 'parameter'_
    partial = 'partial'_
    protected = 'protected'_
    public = 'public'_
    pure = 'pure'_
    record = 'record'_
    redeclare = 'redeclare'_
    replaceable = 'replaceable'_
    return = 'return'_
    stream = 'stream'_
    then = 'then'_
    true = 'true'_
    type = 'type'_
    when = 'when'_
    while = 'while'_
    within = 'within'_
    """)



from parsimonious.nodes import NodeVisitor


class ModelicaPrinter(NodeVisitor):

    def __init__(self):
        print()

    def info(self, node):
        return '\033[94m {:s} \033[0m {:s}'.format(
            node.expr_name, node.full_text[node.start:node.end])

    def generic_visit(self, node, visited_children):
        print(self.info(node))

    def visit_class_definition(self, node, visitied_children):
        print(self.info(node))

    def visit_(self, node, visitied_children):
        pass

    def visit__(self, node, visitied_children):
        pass



#!/usr/bin/env python

import os
import pprint

import ply
import ply.yacc
import ply.lex


# monkey patch ply for python 3 compat since not up on pypi yet
def __getitem__v3_patch(self, n):
    if isinstance(n, slice):
        return [self[i] for i in range(*(n.indices(len(self.slice))))]
    else:  # for integer indices, do what we used to
        if n >= 0:
            return self.slice[n].value
        else:
            return self.stack[n].value
        print(self.classes)
ply.yacc.YaccProduction.__getitem__ = __getitem__v3_patch


class Parser:
    """
    Base class for a lexer/parser that has the rules defined as methods
    """
    tokens = ()
    precedence = ()

    def __init__(self, **kw):
        self.debug = kw.get('debug', 0)
        try:
            modname = os.path.split(
                os.path.splitext(__file__)[0])[1] \
                + "_" + self.__class__.__name__
        except:
            modname = "parser"+"_"+self.__class__.__name__
        self.debugfile = modname + ".dbg"
        self.tabmodule = modname + "_" + "parsetab"
        # print(self.debugfile, self.tabmodule)

        # Build the lexer and parser
        ply.lex.lex(module=self, debug=self.debug)
        ply.yacc.yacc(
            module=self,
            debug=self.debug,
            debugfile=self.debugfile,
            tabmodule=self.tabmodule)

    def run(self):
        while 1:
            try:
                s = raw_input('modelica > ')
            except EOFError:
                break
            if not s:
                continue
            print(ply.yacc.parse(s))

    def parse(self, s):
        res = ply.yacc.parse(s)
        pprint.pprint(res)


class ModelicaParser(Parser):

    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    # Tokens
    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    # ----------------------------------------------------------
    # Reserved Keywords
    # ----------------------------------------------------------
    keywords = [
        'algorithm',
        # 'and',
        # 'annotation',
        # 'assert',
        'block',
        # 'break',
        'class',
        # 'connect',
        'connector',
        'constant',
        # 'constrainedby',
        # 'der',
        'discrete',
        # 'each',
        # 'else',
        # 'elseif',
        # 'elsewhen',
        'encapsulated',
        'end',
        # 'enumeration',
        'equation',
        'expandable',
        # 'extends',
        # 'external',
        # 'false',
        'final',
        'flow',
        # 'for',
        'function',
        # 'if',
        # 'import',
        'impure',
        # 'in',
        # 'initial',
        # 'inner',
        'input',
        # 'loop',
        'model',
        # 'not',
        'operator',
        # 'or',
        # 'outer',
        'output',
        'package',
        'parameter',
        'partial',
        'protected',
        'public',
        'pure',
        'record',
        # 'redeclare',
        # 'return',
        'stream',
        # 'then',
        # 'true',
        'type',
        # 'when',
        # 'while',
        'within',
        ]

    reserved = {key: key.upper() for key in keywords}

    # ----------------------------------------------------------
    # Symbols
    # ----------------------------------------------------------
    symbols = {
        '\;': 'SEMI',
        '\=': 'EQUALS',
        '\+': 'PLUS',
        '\,': 'COMMA',
        # '\(': 'LPAREN',
        # '\)': 'RPAREN',
    }
    # create token handling for symbols
    for key in symbols.keys():
        exec("t_{:s} = '{:s}'".format(symbols[key], key))

    # ----------------------------------------------------------
    # Token Processing
    # ----------------------------------------------------------
    tokens = [
        'IDENT',
        'UNSIGNED_NUMBER',
        'STRING'
    ] + list(reserved.values()) + list(symbols.values())

    t_ignore = ' \t'

    re_dict = {
        'digit': r'[0-9]',
        'nondigit': r'[_a-zA-Z]',
        's_escape': r'([\\][\']|[\\]["])',
        's_char': r'[^\\"]',
    }
    re_process_list = [
        ('q_char', r'{nondigit}|{digit}|\!|\#'),
        ('q_ident', r"'({q_char}|{s_escape})+'"),
        ('ident', r'({nondigit} ({digit}|{nondigit})*) | {q_ident}'),
        ('string', r'\"({s_char}|{s_escape})*\"'),
        ('unsigned_integer', r'{digit}+'),
        ('unsigned_number', r'{unsigned_integer}([\.]{unsigned_integer}?)?'),
    ]
    for key, val in re_process_list:
        re_dict[key] = val.format(**re_dict)

    @ply.lex.TOKEN(re_dict['ident'])
    def t_IDENT(self, t):
        t.type = self.reserved.get(t.value, 'IDENT')
        return t

    @ply.lex.TOKEN(re_dict['unsigned_number'])
    def t_UNSIGNED_NUMBER(self, t):
        return t

    @ply.lex.TOKEN(re_dict['string'])
    def t_STRING(self, t):
        t.value = t.value[1:-1]  # remove double quotes
        return t

    def t_newline(self, t):
        r'\n+'
        t.lexer.lineno += t.value.count("\n")
        return None

    def t_error(self, t):
        print("Illegal character '%s'" % t.value[0])
        t.lexer.skip(1)
        return None

    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
    # Parsing Rules
    # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    # EBNF to BNF
    # {E} -> X = empty | X E
    # [E] -> X = empty | E
    # (E) -> X = E

    precedence = ()

    start = 'stored_definition'

    # ----------------------------------------------------------
    # Parsing Helper Functions
    # ----------------------------------------------------------

    @staticmethod
    def list_extend(p, i1=0, i2=2):
        p[0] = p[i1]
        if p[0] is None:
            p[0] = []
        if len(p) > 2:
            p[0].extend([p[i2]])

    @staticmethod
    def store_as_list(p):
        p[0] = p[1:]

    @staticmethod
    def print_p(self, p):
        for i in range(len(p)):
            print(i, p[i])

    # ----------------------------------------------------------
    # Parsing Helper Rules
    # ----------------------------------------------------------

    def p_empty(self, p):
        'empty : '
        pass

    def p_error(self, p):
        if p is None:
            print("Syntax error at root definition.")
        else:
            print("Syntax error at line {:d}, {:s}".
                  format(p.lineno, p.value))

    # ----------------------------------------------------------
    # B.2.1 Stored Definition - Within
    # ----------------------------------------------------------

    # stored_definition:
    # [ within [ name ] ";" ]
    # { [ final ] class_definition ";" }

    def p_stored_definition(self, p):
        'stored_definition : within_opt class_definitions'
        p[0] = {'within': p[1], 'classes': p[2]}

    def p_within_opt(self, p):
        '''within_opt : WITHIN name_opt SEMI
            | empty'''
        p[0] = {'name': p[1]}

    def p_name_opt(self, p):
        '''name_opt : name
            | empty'''
        p[0] = False if p[1] is None else True

    def p_class_definitions(self, p):
        '''class_definitions : class_definitions final_opt class_definition SEMI
            | empty'''
        if len(p) > 2:  # not empty
            class_defs = p[1]
            final_opt = p[2]
            class_def = p[3]
            if class_defs is None:
                class_defs = {}
            if class_def is not None:
                class_def['final'] = final_opt
                class_defs[class_def['name']] = class_def
            p[0] = class_defs

    def p_final_opt(self, p):
        '''final_opt : FINAL
            | empty'''
        p[0] = False if p[1] is None else True

    # ----------------------------------------------------------
    # B.2.2 Class Definition
    # ----------------------------------------------------------

    # class_definition :
    # [ encapsulated ] class_prefixes
    # class_specifier

    def p_class_definition(self, p):
        'class_definition : encapsulated_opt '\
            'class_prefixes class_specifier'
        encapsulated = p[1]
        prefixes = p[2]
        specifier = p[3]
        p[0] = {
            'encapsulated': encapsulated,
            'name': specifier['name'],
            'comment': specifier['comment'],
            'composition': specifier['composition'],
        }
        for key in prefixes.keys():
            p[0][key] = prefixes[key]

    # class_prefixes :
    # [ partial ]
    # ( class | model | [ operator ] record | block |
    # [ expandable ] connector | type |
    # package | [ ( pure | impure ) ] [ operator ]
    # function | operator )

    def p_class_prefixes(self, p):
        'class_prefixes : partial_opt class_type_opt'
        p[0] = {'partial': p[1], 'type': p[2]}

    def p_partial_opt(self, p):
        '''partial_opt : PARTIAL
            | empty'''
        p[0] = False if p[1] is None else True

    def p_class_type_opt(self, p):
        '''class_type_opt : CLASS
            | MODEL
            | RECORD
            | OPERATOR RECORD
            | BLOCK
            | CONNECTOR
            | EXPANDABLE CONNECTOR
            | TYPE
            | PACKAGE
            | PURE FUNCTION
            | IMPURE FUNCTION
            | PURE OPERATOR FUNCTION
            | IMPURE OPERATOR FUNCTION
            | OPERATOR'''
        p[0] = p[1]  # ''.join(p[1:len(p)])

    # class_specifier :
    # IDENT string_comment composition end IDENT
    # | IDENT "=" base_prefix name [ array_subscripts ]
    # [ class_modification ] comment
    # | IDENT "=" enumeration "(" ( [enum_list] | ":" ) ")" comment
    # | IDENT "=" der "(" name "," IDENT { "," IDENT } ")" comment
    # | extends IDENT [ class_modification ] string_comment composition
    # end IDENT

    # TODO
    def p_class_specifier(self, p):
        'class_specifier : IDENT string_comment_opt '\
            'composition END IDENT'
        name = p[1]
        comment = p[2]
        composition = p[3]
        name_end = p[5]
        if name != name_end:
            raise IOError(
                "Syntax error at line {:d}, "
                "class names don't match at beginning and end: "
                "{:s} {:s}".
                format(p.lineno(6), name, name_end))
        p[0] = {
            'name': name,
            'comment': comment,
            'composition': composition,
        }

    # base_prefix :
    # type_prefix

    # def p_base_prefix(self, p):
    #     'base_prefix : type_prefix'

    # enum_list
    # : enumeration_literal { "," enumeration_literal}
    # enumeration_literal : IDENT comment

    # composition :
    # element_list
    # { public element_list |
    # protected element_list |
    # equation_section |
    # algorithm_section
    # }
    # [ external [ language_specification ]
    # [ external_function_call ] [ annotation ] ";" ]
    # [ annotation ";" ]

    def p_composition(self, p):
        '''composition : element_list composition_list'''
        self.list_extend(p)

    def p_composition_or(self, p):
        '''composition_or : PUBLIC element_list
            | PROTECTED element_list
            | equation_section
            | algorithm_section'''
        self.store_as_list(p)

    def p_composition_list(self, p):
        '''composition_list : composition_list composition_or
            | empty'''
        self.list_extend(p)

    # language_specification :
    # STRING

    # def p_language_specification(self, p):
    #     'language_specification : STRING'
    #     self.store_as_list(p)

    # external_function_call :
    # [ component_reference "=" ]
    # IDENT "(" [ expression_list ] ")"

    # element_list :
    # { element ";" }

    def p_element_list(self, p):
        '''element_list : element_list element SEMI
            | empty'''
        self.store_as_list(p)

    # element :
    # import_clause |
    # extends_clause |
    # [ redeclare ]
    # [ final ]
    # [ inner ] [ outer ]
    # ( ( class_definition | component_clause) |
    # replaceable ( class_definition | component_clause)247
    # [constraining_clause comment])

    def p_element(self, p):
        'element : component_clause'
        p[0] = p[1]

    # import_clause :
    # import ( IDENT "=" name | name ["." ( "*" |
    # "{" import_list "}" ) ] ) comment
    # import_list :
    # IDENT [ "," import_list ]

    # def p_external_function_call(self, p):
    #     'external_function_call : component_reference_opt '\
    #         ' IDENT "(" expression_list ")"'
    #     self.store_as_list(p)

    # ----------------------------------------------------------
    # B.2.3 Extends
    # ----------------------------------------------------------

    # extends_clause :
    # extends name [ class_modification ] [annotation]

    # constraining_clause :
    # constrainedby name [ class_modification ]

    # ----------------------------------------------------------
    # B.2.4 Component Clause
    # ----------------------------------------------------------

    # component_clause:
    # type_prefix type_specifier [ array_subscripts ] component_list

    # type_prefix :
    # [ flow | stream ]
    # [ discrete | parameter | constant ] [ input | output ]
    def p_type_prefix(self, p):
        'type_prefix : type_prefix_1 type_prefix_2 type_prefix_3'
        p[0] = [p[1], p[2], p[3]]

    def p_type_prefix1(self, p):
        '''type_prefix_1 : FLOW
            | STREAM
            | empty'''
        p[0] = p[1]

    def p_type_prefix2(self, p):
        '''type_prefix_2 : DISCRETE
            | PARAMETER
            | CONSTANT
            | empty'''
        p[0] = p[1]

    def p_type_prefix3(self, p):
        '''type_prefix_3 : INPUT
            | OUTPUT
            | empty'''
        p[0] = p[1]

    # type_specifier :
    # name

    def p_type_specifier(self, p):
        'type_specifier : name'
        p[0] = p[1]

    # component_list :
    # component_declaration { "," component_declaration }

    # component_declaration :
    # declaration [ condition_attribute ] comment

    # condition_attribute:
    # if expression

    # declaration :
    # IDENT [ array_subscripts ] [ modification ]

    # ----------------------------------------------------------
    # B.2.5 Modification
    # ----------------------------------------------------------

    # modification :
    # class_modification [ "=" expression ]
    # | "=" expression
    # | ":=" expression

    # class_modification :
    # "(" [ argument_list ] ")"

    # def p_class_modification(self, p):
    #     'class_modification : LPAREN argument_list_opt RPAREN'
    #     p[0] = p[2]

    # argument_list :
    # argument { "," argument }

    # def p_arguemnt_list(self, p):
    #     'argument_list : argument arguement_list_part'
    #     p[0] = p[1:]
    # def p_argument_list_part(self, p):
    #     '''argument_list_part : argument_list_part COMMA argument
    #         | empty'''
    #     p[0] = p[1]

    # argument :
    # element_modification_or_replaceable
    # | element_redeclaration

    # TODO
    # def p_argument(self, p):
    #     'argument : IDENT'

    # element_modification_or_replaceable:
    # [ each ] [ final ] ( element_modification | element_replaceable)

    # element_modification :
    # name [ modification ] string_comment

    # element_redeclaration :
    # redeclare [ each ] [ final ]
    # ( ( short_class_definition | component_clause1) | element_replaceable )

    # element_replaceable:
    # replaceable ( short_class_definition | component_clause1)
    # [constraining_clause]

    # component_clause1 :
    # type_prefix type_specifier component_declaration1

    # component_declaration1 :
    # declaration comment

    # short_class_definition :
    # class_prefixes IDENT "="
    # ( base_prefix name [ array_subscripts ]
    # [ class_modification ] comment |
    # enumeration "(" ( [enum_list] | ":" ) ")" comment )

    # ----------------------------------------------------------
    # B.2.6 Equations
    # ----------------------------------------------------------

    # equation_section :
    # [ initial ] equation { equation ";" }

    # TODO
    def p_equation_section(self, p):
        'equation_section : EQUATION'
        self.store_as_list(p)

    # algorithm_section :
    # [ initial ] algorithm { statement ";" }

    # TODO
    def p_algorithm_section(self, p):
        'algorithm_section : ALGORITHM'
        self.store_as_list(p)

    # equation :
    # ( simple_expression "=" expression
    # | if_equation
    # | for_equation
    # | connect_clause
    # | when_equation
    # | name function_call_args )
    # comment

    # statement :
    # ( component_reference ( ":=" expression | function_call_args )
    # | "(" output_expression_list ")" ":="
    # component_reference function_call_args
    # | break
    # | return
    # | if_statement
    # | for_statement
    # | while_statement
    # | when_statement )
    # comment

    # if_equation :
    # if expression then
    # { equation ";" }
    # { elseif expression then
    # { equation ";" }
    # }
    # [ else
    # { equation ";" }
    # ]
    # end if

    # if_statement :
    # if expression then
    # { statement ";" }
    # { elseif expression then
    # { statement ";" }
    # }
    # [ else
    # { statement ";" }
    # ]
    # end if

    # for_equation :
    # for for_indices loop
    # { equation ";" }
    # end for

    # for_statement :
    # for for_indices loop
    # { statement ";" }
    # end for

    # for_indices :
    # for_index {"," for_index}

    # for_index:
    # IDENT [ in expression ]

    # while_statement :
    # while expression loop
    # { statement ";" }
    # end while

    # when_equation :
    # when expression then
    # { equation ";" }
    # { elsewhen expression then
    # { equation ";" } }
    # end when

    # when_statement :
    # when expression then
    # { statement ";" }
    # { elsewhen expression then
    # { statement ";" } }
    # end when

    # connect_clause :
    # connect "(" component_reference "," component_reference ")"

    def p_component_clause(self, p):
        'component_clause : type_prefix type_specifier '\
            'array_subscripts_opt component_list'
        p[0] = {
            'prefix': p[1],
            'specifier': p[2],
            'array_subscripts': p[3],
            'component_list': p[4],
        }

    # TODO
    def p_array_subscripts_opt(self, p):
        '''array_subscripts_opt : empty'''
        p[0] = p[1]

    def p_component_list(self, p):
        '''component_list : component
            | component_list COMMA component
            | empty
        '''
        if len(p) > 2:
            if p[1] is None:
                p[0] = []
            else:
                p[0] = p[1]
            p[0].extend([p[3]])
        elif p[1] is not None:
            p[0] = [p[1]]

    def p_component(self, p):
        '''component : IDENT EQUALS UNSIGNED_NUMBER'''
        p[0] = [p[1], p[2], p[3]]

    def p_encapsulated_opt(self, p):
        '''encapsulated_opt : ENCAPSULATED
            | empty'''
        p[0] = False if p[1] is None else True

    # ----------------------------------------------------------
    # B.2.7 Expressions
    # ----------------------------------------------------------

    # expression :
    # simple_expression
    # | if expression then expression { elseif expression then expression }
    # else expression

    # simple_expression :
    # logical_expression [ ":" logical_expression [ ":" logical_expression ] ]

    # logical_expression :
    # logical_term { or logical_term }

    # logical_term :
    # logical_factor { and logical_factor }

    # logical_factor :
    # [ not ] relation

    # relation :
    # arithmetic_expression [ rel_op arithmetic_expression ]

    # rel_op :
    # "<" | "<=" | ">" | ">=" | "==" | "<>"

    # arithmetic_expression :
    # [ add_op ] term { add_op term }

    # add_op :
    # "+" | "-" | ".+" | ".-"

    # term :
    # factor { mul_op factor }

    # mul_op :
    # "*" | "/" | ".*" | "./"

    # factor :
    # primary [ ("^" | ".^") primary ]

    # primary :
    # UNSIGNED_NUMBER
    # | STRING
    # | false
    # | true
    # | ( name | der | initial ) function_call_args
    # | component_reference
    # | "(" output_expression_list ")"
    # | "[" expression_list { ";" expression_list } "]"
    # | "{" function_arguments "}"
    # | end

    # name :
    # [ "." ] IDENT { "." IDENT }

    # TODO
    def p_name(self, p):
        '''name : IDENT'''
        self.store_as_list(p)

    # TODO
    # def p_names(self, p):
    #     '''names : names '.' IDENT
    #         | empty'''
    #     self.store_as_list(p)

    # component_reference :
    # [ "." ] IDENT [ array_subscripts ] { "." IDENT [ array_subscripts ] }

    # def p_component_reference_opt(self, p):
    #     '''component_refefrence_opt : component_reference EQUALS
    #         | empty'''
    #     self.store_as_list(p)

    # TODO
    # def p_component_reference(self, p):
    #     '''component_reference : IDENT'''
    #     self.store_as_list(p)

    # function_call_args :
    # "(" [ function_arguments ] ")"

    # function_arguments :
    # function_argument [ "," function_arguments | for for_indices ]
    # | named_arguments

    # named_arguments: named_argument [ "," named_arguments ]

    # named_argument: IDENT "=" function_argument

    # function_argument :
    # function name "(" [ named_arguments ] ")" | expression
    # output_expression_list:
    # [ expression ] { "," [ expression ] }

    # expression_list :
    # expression { "," expression }

    # array_subscripts :
    # "[" subscript { "," subscript } "]"

    # subscript :
    # ":" | expression

    # comment :
    # string_comment [ annotation ]

    # string_comment :
    # [ STRING { "+" STRING } ]

    def p_string_comment_part(self, p):
        '''string_comment_part : string_comment_part PLUS STRING
            | empty'''
        if (len(p) > 2):
            if p[1] is None:
                p[0] = p[3]
            else:
                p[0] = ''.join([p[1], p[3]])

    def p_string_comment(self, p):
        'string_comment : STRING string_comment_part'
        p[0] = ''.join([p[1], p[2]])

    def p_string_comment_opt(self, p):
        '''string_comment_opt : string_comment
            | empty'''
        p[0] = p[1]

    # annotation :
    # annotation class_modification
    # def p_annotation(self, p):
    #     'annotation : STRING'
    #     p[0] = p[1]

if __name__ == '__main__':
    parser = ModelicaParser()
    parser.parse('''
    class hello1 "hello" + " bye"
    end hello1;
    class hello2 "hello world" + " example"
        flow discrete input Real a=1, b=2;
    public
        Real c=3;
        Real d=3;
    equation
    algorithm
    end hello2;
    ''')




# THIS FILE IS GENERATED FROM SETUP.PY
short_version = '0.0.3'
version = '0.0.3'
full_version = '0.0.3'
git_revision = '1a8de7b3dfa7476abecb972122ff83abc4ba9974'
release = True

if not release:
    version = full_version



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# CAVEAT UTILITOR
# This file was automatically generated by Grako.
#    https://bitbucket.org/apalala/grako/
# Any changes you make to it will be overwritten the
# next time the file is generated.
#

from __future__ import print_function, division, absolute_import, unicode_literals
from grako.parsing import * # noqa
from grako.exceptions import * # noqa


__version__ = '14.160.22.15.56'


class ModelicaParser(Parser):
    def __init__(self, whitespace=None, **kwargs):
        super(ModelicaParser, self).__init__(whitespace=whitespace, **kwargs)

    @rule_def
    def _stored_definition_(self):
        with self._optional():
            self._within_()
            with self._optional():
                self._name_()
            self._token(';')
        def block0():
            with self._optional():
                self._final_()
            self._class_definition_()
            self._token(';')
        self._closure(block0)

    @rule_def
    def _class_definition_(self):
        with self._optional():
            self._encapsulated_()
        self._class_prefixes_()
        self._class_specifier_()

    @rule_def
    def _class_prefixes_(self):
        with self._optional():
            self._partial_()
        with self._group():
            with self._choice():
                with self._option():
                    self._class_()
                with self._option():
                    self._model_()
                with self._option():
                    with self._optional():
                        self._operator_()
                    self._record_()
                with self._option():
                    self._block_()
                with self._option():
                    with self._optional():
                        self._expandable_()
                    self._connector_()
                with self._option():
                    self._type_()
                with self._option():
                    self._package_()
                with self._option():
                    with self._optional():
                        with self._group():
                            with self._choice():
                                with self._option():
                                    self._pure_()
                                with self._option():
                                    self._impure_()
                                self._error('no available options')
                    with self._optional():
                        self._operator_()
                    self._function_()
                with self._option():
                    self._operator_()
                self._error('no available options')

    @rule_def
    def _class_specifier_(self):
        self._IDENT_()
        with self._optional():
            self._string_comment_()
        self._composition_()
        self._end_()
        self._IDENT_()

    @rule_def
    def _base_prefix_(self):
        self._type_prefix_()

    @rule_def
    def _enum_list_(self):
        self._enumeration_literal_()
        def block0():
            self._token(',')
            self._enumeration_literal_()
        self._closure(block0)

    @rule_def
    def _enumeration_literal_(self):
        self._IDENT_()
        self._comment_()

    @rule_def
    def _composition_(self):
        self._element_list_()

    @rule_def
    def _language_specification_(self):
        self._STRING_()

    @rule_def
    def _external_function_call_(self):
        with self._optional():
            self._component_reference_()
            self._token('=')
        self._IDENT_()
        self._token('(')
        with self._optional():
            self._expression_list_()
        self._token(')')

    @rule_def
    def _element_list_(self):
        def block0():
            self._element_()
            self._token(';')
        self._closure(block0)

    @rule_def
    def _element_(self):
        self._component_clause_()

    @rule_def
    def _import_clause_(self):
        self._import_()
        with self._group():
            with self._choice():
                with self._option():
                    self._IDENT_()
                    self._token('=')
                    self._name_()
                with self._option():
                    self._name_()
                    with self._optional():
                        self._token('.')
                        with self._group():
                            with self._choice():
                                with self._option():
                                    self._token('*')
                                with self._option():
                                    self._token('{')
                                    self._import_list_()
                                    self._token('}')
                                self._error('expecting one of: *')
                self._error('no available options')
        self._comment_()

    @rule_def
    def _import_list_(self):
        self._IDENT_()
        with self._optional():
            self._token(',')
            self._import_list_()

    @rule_def
    def _extends_clause_(self):
        self._extends_()
        self._name_()
        with self._optional():
            self._class_modification_()
        with self._optional():
            self._annotation_()

    @rule_def
    def _constraining_clause_(self):
        self._constrainedby_()
        self._name_()
        with self._optional():
            self._class_modification_()

    @rule_def
    def _component_clause_(self):
        self._type_prefix_()
        self._IDENT_()

    @rule_def
    def _type_prefix_(self):
        with self._optional():
            with self._choice():
                with self._option():
                    self._flow_()
                with self._option():
                    self._stream_()
                self._error('no available options')
        with self._optional():
            with self._choice():
                with self._option():
                    self._discrete_()
                with self._option():
                    self._parameter_()
                with self._option():
                    self._constant_()
                self._error('no available options')
        with self._optional():
            with self._choice():
                with self._option():
                    self._input_()
                with self._option():
                    self._output_()
                self._error('no available options')

    @rule_def
    def _type_specifier_(self):
        self._IDENT_()

    @rule_def
    def _component_list_(self):
        self._component_declaration_()
        def block0():
            self._token(',')
            self._component_declaration_()
        self._closure(block0)

    @rule_def
    def _component_declaration_(self):
        self._declaration_()
        with self._optional():
            self._condition_attribute_()
        self._comment_()

    @rule_def
    def _condition_attribute_(self):
        self._if_()
        self._expression_()

    @rule_def
    def _declaration_(self):
        self._IDENT_()
        with self._optional():
            self._array_subscripts_()
        with self._optional():
            self._modification_()

    @rule_def
    def _modification_(self):
        with self._choice():
            with self._option():
                self._class_modification_()
                with self._optional():
                    self._token('=')
                    self._expression_()
            with self._option():
                self._token('=')
                self._expression_()
            with self._option():
                self._token(':=')
                self._expression_()
            self._error('no available options')

    @rule_def
    def _class_modification_(self):
        self._token('(')
        with self._optional():
            self._argument_list_()
        self._token(')')

    @rule_def
    def _argument_list_(self):
        self._argument_()
        def block0():
            self._token(',')
            self._argument_()
        self._closure(block0)

    @rule_def
    def _argument_(self):
        with self._choice():
            with self._option():
                self._element_modification_or_replaceable_()
            with self._option():
                self._element_redeclaration_()
            self._error('no available options')

    @rule_def
    def _element_modification_or_replaceable_(self):
        with self._optional():
            self._each_()
        with self._optional():
            self._final_()
        with self._group():
            with self._choice():
                with self._option():
                    self._element_modification_()
                with self._option():
                    self._element_replaceable_()
                self._error('no available options')

    @rule_def
    def _element_modification_(self):
        self._name_()
        with self._optional():
            self._modification_()
        self._string_comment_()

    @rule_def
    def _element_redeclaration_(self):
        self._redeclare_()
        with self._optional():
            self._each_()
        with self._optional():
            self._final_()
        with self._group():
            with self._choice():
                with self._option():
                    with self._group():
                        with self._choice():
                            with self._option():
                                self._short_class_definition_()
                            with self._option():
                                self._component_clause1_()
                            self._error('no available options')
                with self._option():
                    self._element_replaceable_()
                self._error('no available options')

    @rule_def
    def _element_replaceable_(self):
        self._replaceable_()
        with self._group():
            with self._choice():
                with self._option():
                    self._short_class_definition_()
                with self._option():
                    self._component_clause1_()
                self._error('no available options')
        with self._optional():
            self._constraining_clause_()

    @rule_def
    def _component_clause1_(self):
        self._type_prefix_()
        self._type_specifier_()
        self._component_declaration1_()

    @rule_def
    def _component_declaration1_(self):
        self._declaration_()
        self._comment_()

    @rule_def
    def _short_class_definition_(self):
        self._class_prefixes_()
        self._IDENT_()
        self._token('=')
        with self._group():
            with self._choice():
                with self._option():
                    self._base_prefix_()
                    self._name_()
                    with self._optional():
                        self._array_subscripts_()
                    with self._optional():
                        self._class_modification_()
                    self._comment_()
                with self._option():
                    self._enumeration_()
                    self._token('(')
                    with self._group():
                        with self._choice():
                            with self._option():
                                with self._optional():
                                    self._enum_list_()
                            with self._option():
                                self._token(':')
                            self._error('expecting one of: :')
                    self._token(')')
                    self._comment_()
                self._error('no available options')

    @rule_def
    def _equation_section_(self):
        with self._optional():
            self._initial_()
        self._equation_()
        def block0():
            self._equation_()
            self._token(';')
        self._closure(block0)

    @rule_def
    def _algorithm_section_(self):
        with self._optional():
            self._initial_()
        self._algorithm_()
        def block0():
            self._statement_()
            self._token(';')
        self._closure(block0)

    @rule_def
    def _equation_(self):
        with self._choice():
            with self._option():
                with self._group():
                    with self._choice():
                        with self._option():
                            self._simple_expression_()
                            self._token('=')
                            self._expression_()
                        with self._option():
                            self._if_equation_()
                        with self._option():
                            self._for_equation_()
                        with self._option():
                            self._connect_clause_()
                        with self._option():
                            self._when_equation_()
                        with self._option():
                            self._name_()
                            self._function_call_args_()
                        self._error('no available options')
                self._comment_()
            with self._option():
                self._token('equation')
            self._error('expecting one of: equation')

    @rule_def
    def _statement_(self):
        with self._group():
            with self._choice():
                with self._option():
                    self._component_reference_()
                    with self._group():
                        with self._choice():
                            with self._option():
                                self._token(':=')
                                self._expression_()
                            with self._option():
                                self._function_call_args_()
                            self._error('no available options')
                with self._option():
                    self._token('(')
                    self._output_expression_list_()
                    self._token(')')
                    self._token(':=')
                    self._component_reference_()
                    self._function_call_args_()
                with self._option():
                    self._break_()
                with self._option():
                    self._return_()
                with self._option():
                    self._if_statement_()
                with self._option():
                    self._for_statement_()
                with self._option():
                    self._while_statement_()
                with self._option():
                    self._when_statement_()
                self._error('no available options')
        self._comment_()

    @rule_def
    def _if_equation_(self):
        self._if_()
        self._expression_()
        self._then_()
        def block0():
            self._equation_()
            self._token(';')
        self._closure(block0)
        def block1():
            self._elseif_()
            self._expression_()
            self._then_()
            def block2():
                self._equation_()
                self._token(';')
            self._closure(block2)
        self._closure(block1)
        with self._optional():
            self._else_()
            def block3():
                self._equation_()
                self._token(';')
            self._closure(block3)
        self._end_()
        self._if_()

    @rule_def
    def _if_statement_(self):
        self._if_()
        self._expression_()
        self._then_()
        def block0():
            self._statement_()
            self._token(';')
        self._closure(block0)
        def block1():
            self._elseif_()
            self._expression_()
            self._then_()
            def block2():
                self._statement_()
                self._token(';')
            self._closure(block2)
        self._closure(block1)
        with self._optional():
            self._else_()
            def block3():
                self._statement_()
                self._token(';')
            self._closure(block3)
        self._end_()
        self._if_()

    @rule_def
    def _for_equation_(self):
        self._for_()
        self._for_indices_()
        self._loop_()
        def block0():
            self._equation_()
            self._token(';')
        self._closure(block0)
        self._end_()
        self._for_()

    @rule_def
    def _for_statement_(self):
        self._for_()
        self._for_indices_()
        self._loop_()
        def block0():
            self._statement_()
            self._token(';')
        self._closure(block0)
        self._end_()
        self._for_()

    @rule_def
    def _for_indices_(self):
        self._for_index_()
        def block0():
            self._token(',')
            self._for_index_()
        self._closure(block0)

    @rule_def
    def _for_index_(self):
        self._IDENT_()
        with self._optional():
            self._in_()
            self._expression_()

    @rule_def
    def _while_statement_(self):
        self._while_()
        self._expression_()
        self._loop_()
        def block0():
            self._statement_()
            self._token(';')
        self._closure(block0)
        self._end_()
        self._while_()

    @rule_def
    def _when_equation_(self):
        self._when_()
        self._expression_()
        self._then_()
        def block0():
            self._equation_()
            self._token(';')
        self._closure(block0)
        def block1():
            self._elsewhen_()
            self._expression_()
            self._then_()
            def block2():
                self._equation_()
                self._token(';')
            self._closure(block2)
        self._closure(block1)
        self._end_()
        self._when_()

    @rule_def
    def _when_statement_(self):
        self._when_()
        self._expression_()
        self._then_()
        def block0():
            self._statement_()
            self._token(';')
        self._closure(block0)
        def block1():
            self._elsewhen_()
            self._expression_()
            self._then_()
            def block2():
                self._statement_()
                self._token(';')
            self._closure(block2)
        self._closure(block1)
        self._end_()
        self._when_()

    @rule_def
    def _connect_clause_(self):
        self._connect_()
        self._token('(')
        self._component_reference_()
        self._token(',')
        self._component_reference_()
        self._token(')')

    @rule_def
    def _expression_(self):
        with self._choice():
            with self._option():
                self._simple_expression_()
            with self._option():
                self._if_()
                self._expression_()
                self._then_()
                self._expression_()
                def block0():
                    self._elseif_()
                    self._expression_()
                    self._then_()
                    self._expression_()
                self._closure(block0)
                self._else_()
                self._expression_()
            self._error('no available options')

    @rule_def
    def _simple_expression_(self):
        self._logical_expression_()
        with self._optional():
            self._token(':')
            self._logical_expression_()
            with self._optional():
                self._token(':')
                self._logical_expression_()

    @rule_def
    def _logical_expression_(self):
        self._logical_term_()
        def block0():
            self._or_()
            self._logical_term_()
        self._closure(block0)

    @rule_def
    def _logical_term_(self):
        self._logical_factor_()
        def block0():
            self._and_()
            self._logical_factor_()
        self._closure(block0)

    @rule_def
    def _logical_factor_(self):
        with self._optional():
            self._not_()
        self._relation_()

    @rule_def
    def _relation_(self):
        self._arithmetic_expression_()
        with self._optional():
            self._rel_op_()
            self._arithmetic_expression_()

    @rule_def
    def _rel_op_(self):
        with self._choice():
            with self._option():
                self._token('<')
            with self._option():
                self._token('<=')
            with self._option():
                self._token('>')
            with self._option():
                self._token('>=')
            with self._option():
                self._token('==')
            with self._option():
                self._token('<>')
            self._error('expecting one of: > >= <> <= < ==')

    @rule_def
    def _arithmetic_expression_(self):
        with self._optional():
            self._add_op_()
        self._term_()
        def block0():
            self._add_op_()
            self._term_()
        self._closure(block0)

    @rule_def
    def _add_op_(self):
        with self._choice():
            with self._option():
                self._token('+')
            with self._option():
                self._token('-')
            with self._option():
                self._token('.+')
            with self._option():
                self._token('.-')
            self._error('expecting one of: + .- .+ -')

    @rule_def
    def _term_(self):
        self._factor_()
        def block0():
            self._mul_op_()
            self._factor_()
        self._closure(block0)

    @rule_def
    def _mul_op_(self):
        with self._choice():
            with self._option():
                self._token('*')
            with self._option():
                self._token('/')
            with self._option():
                self._token('.*')
            with self._option():
                self._token('./')
            self._error('expecting one of: ./ .* * /')

    @rule_def
    def _factor_(self):
        self._primary_()
        with self._optional():
            with self._group():
                with self._choice():
                    with self._option():
                        self._token('^')
                    with self._option():
                        self._token('.^')
                    self._error('expecting one of: ^ .^')
            self._primary_()

    @rule_def
    def _primary_(self):
        with self._choice():
            with self._option():
                self._UNSIGNED_NUMBER_()
            with self._option():
                self._STRING_()
            with self._option():
                self._false_()
            with self._option():
                self._true_()
            with self._option():
                with self._group():
                    with self._choice():
                        with self._option():
                            self._name_()
                        with self._option():
                            self._der_()
                        with self._option():
                            self._initial_()
                        self._error('no available options')
                self._function_call_args_()
            with self._option():
                self._component_reference_()
            with self._option():
                self._token('(')
                self._output_expression_list_()
                self._token(')')
            with self._option():
                self._token('[')
                self._expression_list_()
                def block1():
                    self._token(';')
                    self._expression_list_()
                self._closure(block1)
                self._token(']')
            with self._option():
                self._token('{')
                self._function_arguments_()
                self._token('}')
            with self._option():
                self._end_()
            self._error('no available options')

    @rule_def
    def _name_(self):
        self._IDENT_()

    @rule_def
    def _component_reference_(self):
        with self._optional():
            self._token('.')
        self._IDENT_()
        with self._optional():
            self._array_subscripts_()
        def block0():
            self._token('.')
            self._IDENT_()
            with self._optional():
                self._array_subscripts_()
        self._closure(block0)

    @rule_def
    def _function_call_args_(self):
        self._token('(')
        with self._optional():
            self._function_arguments_()
        self._token(')')

    @rule_def
    def _function_arguments_(self):
        with self._choice():
            with self._option():
                self._function_argument_()
                with self._optional():
                    with self._choice():
                        with self._option():
                            self._token(',')
                            self._function_arguments_()
                        with self._option():
                            self._for_()
                            self._for_indices_()
                        self._error('no available options')
            with self._option():
                self._named_arguments_()
            self._error('no available options')

    @rule_def
    def _named_arguments_(self):
        self._named_argument_()
        with self._optional():
            self._token(',')
            self._named_arguments_()

    @rule_def
    def _named_argument_(self):
        self._IDENT_()
        self._token('=')
        self._function_argument_()

    @rule_def
    def _function_argument_(self):
        with self._choice():
            with self._option():
                self._function_()
                self._name_()
                self._token('(')
                with self._optional():
                    self._named_arguments_()
                self._token(')')
            with self._option():
                self._expression_()
            self._error('no available options')

    @rule_def
    def _output_expression_list_(self):
        with self._optional():
            self._expression_()
        def block0():
            self._token(',')
            with self._optional():
                self._expression_()
        self._closure(block0)

    @rule_def
    def _expression_list_(self):
        self._expression_()
        def block0():
            self._token(',')
            self._expression_()
        self._closure(block0)

    @rule_def
    def _array_subscripts_(self):
        self._token('[')
        self._subscript_()
        def block0():
            self._token(',')
            self._subscript_()
        self._closure(block0)
        self._token(']')

    @rule_def
    def _subscript_(self):
        with self._choice():
            with self._option():
                self._token(':')
            with self._option():
                self._expression_()
            self._error('expecting one of: :')

    @rule_def
    def _comment_(self):
        self._string_comment_()
        with self._optional():
            self._annotation_()

    @rule_def
    def _string_comment_(self):
        with self._optional():
            self._STRING_()
            def block0():
                self._token('+')
                self._STRING_()
            self._closure(block0)

    @rule_def
    def _annotation_(self):
        with self._choice():
            with self._option():
                self._annotation_()
                self._class_modification_()
            with self._option():
                self._token('annotation')
            self._error('expecting one of: annotation')

    @rule_def
    def _algorithm_(self):
        self._token('algorithm')

    @rule_def
    def _and_(self):
        self._token('and')

    @rule_def
    def _assert_(self):
        self._token('assert')

    @rule_def
    def _block_(self):
        self._token('block')

    @rule_def
    def _break_(self):
        self._token('break')

    @rule_def
    def _class_(self):
        self._token('class')

    @rule_def
    def _connect_(self):
        self._token('connect')

    @rule_def
    def _connector_(self):
        self._token('connector')

    @rule_def
    def _constant_(self):
        self._token('constant')

    @rule_def
    def _constrainedby_(self):
        self._token('constrainedby')

    @rule_def
    def _der_(self):
        self._token('der')

    @rule_def
    def _discrete_(self):
        self._token('discrete')

    @rule_def
    def _each_(self):
        self._token('each')

    @rule_def
    def _else_(self):
        self._token('else')

    @rule_def
    def _elseif_(self):
        self._token('elseif')

    @rule_def
    def _elsewhen_(self):
        self._token('elsewhen')

    @rule_def
    def _encapsulated_(self):
        self._token('encapsulated')

    @rule_def
    def _end_(self):
        self._token('end')

    @rule_def
    def _enumeration_(self):
        self._token('enumeration')

    @rule_def
    def _expandable_(self):
        self._token('expandable')

    @rule_def
    def _extends_(self):
        self._token('extends')

    @rule_def
    def _external_(self):
        self._token('external')

    @rule_def
    def _false_(self):
        self._token('false')

    @rule_def
    def _final_(self):
        self._token('final')

    @rule_def
    def _flow_(self):
        self._token('flow')

    @rule_def
    def _for_(self):
        self._token('for')

    @rule_def
    def _function_(self):
        self._token('function')

    @rule_def
    def _if_(self):
        self._token('if')

    @rule_def
    def _import_(self):
        self._token('import')

    @rule_def
    def _impure_(self):
        self._token('impure')

    @rule_def
    def _in_(self):
        self._token('in')

    @rule_def
    def _initial_(self):
        self._token('initial')

    @rule_def
    def _inner_(self):
        self._token('inner')

    @rule_def
    def _input_(self):
        self._token('input')

    @rule_def
    def _loop_(self):
        self._token('loop')

    @rule_def
    def _model_(self):
        self._token('model')

    @rule_def
    def _not_(self):
        self._token('not')

    @rule_def
    def _operator_(self):
        self._token('operator')

    @rule_def
    def _or_(self):
        self._token('or')

    @rule_def
    def _outer_(self):
        self._token('outer')

    @rule_def
    def _output_(self):
        self._token('output')

    @rule_def
    def _package_(self):
        self._token('package')

    @rule_def
    def _parameter_(self):
        self._token('parameter')

    @rule_def
    def _partial_(self):
        self._token('partial')

    @rule_def
    def _protected_(self):
        self._token('protected')

    @rule_def
    def _public_(self):
        self._token('public')

    @rule_def
    def _pure_(self):
        self._token('pure')

    @rule_def
    def _record_(self):
        self._token('record')

    @rule_def
    def _redeclare_(self):
        self._token('redeclare')

    @rule_def
    def _replaceable_(self):
        self._token('replaceable')

    @rule_def
    def _return_(self):
        self._token('return')

    @rule_def
    def _stream_(self):
        self._token('stream')

    @rule_def
    def _then_(self):
        self._token('then')

    @rule_def
    def _true_(self):
        self._token('true')

    @rule_def
    def _type_(self):
        self._token('type')

    @rule_def
    def _when_(self):
        self._token('when')

    @rule_def
    def _while_(self):
        self._token('while')

    @rule_def
    def _within_(self):
        self._token('within')

    @rule_def
    def _re_IDENT_(self):
        self._pattern(r'[_a-zA-Z][_a-zA-Z0-9]*')

    @rule_def
    def _re_STRING_(self):
        self._pattern(r'"[^"]+"')

    @rule_def
    def _re_UNSIGNED_NUMBER_(self):
        self._pattern(r'[0-9]+([\.] [0-9]+)?[[eE][\+\-]?[0-9]+')

    @rule_def
    def _IDENT_(self):
        self._re_IDENT_()

    @rule_def
    def _STRING_(self):
        self._re_STRING_()

    @rule_def
    def _UNSIGNED_NUMBER_(self):
        self._re_UNSIGNED_NUMBER_()


class ModelicaSemanticParser(CheckSemanticsMixin, ModelicaParser):
    pass


class ModelicaSemantics(object):
    def stored_definition(self, ast):
        return ast

    def class_definition(self, ast):
        return ast

    def class_prefixes(self, ast):
        return ast

    def class_specifier(self, ast):
        return ast

    def base_prefix(self, ast):
        return ast

    def enum_list(self, ast):
        return ast

    def enumeration_literal(self, ast):
        return ast

    def composition(self, ast):
        return ast

    def language_specification(self, ast):
        return ast

    def external_function_call(self, ast):
        return ast

    def element_list(self, ast):
        return ast

    def element(self, ast):
        return ast

    def import_clause(self, ast):
        return ast

    def import_list(self, ast):
        return ast

    def extends_clause(self, ast):
        return ast

    def constraining_clause(self, ast):
        return ast

    def component_clause(self, ast):
        return ast

    def type_prefix(self, ast):
        return ast

    def type_specifier(self, ast):
        return ast

    def component_list(self, ast):
        return ast

    def component_declaration(self, ast):
        return ast

    def condition_attribute(self, ast):
        return ast

    def declaration(self, ast):
        return ast

    def modification(self, ast):
        return ast

    def class_modification(self, ast):
        return ast

    def argument_list(self, ast):
        return ast

    def argument(self, ast):
        return ast

    def element_modification_or_replaceable(self, ast):
        return ast

    def element_modification(self, ast):
        return ast

    def element_redeclaration(self, ast):
        return ast

    def element_replaceable(self, ast):
        return ast

    def component_clause1(self, ast):
        return ast

    def component_declaration1(self, ast):
        return ast

    def short_class_definition(self, ast):
        return ast

    def equation_section(self, ast):
        return ast

    def algorithm_section(self, ast):
        return ast

    def equation(self, ast):
        return ast

    def statement(self, ast):
        return ast

    def if_equation(self, ast):
        return ast

    def if_statement(self, ast):
        return ast

    def for_equation(self, ast):
        return ast

    def for_statement(self, ast):
        return ast

    def for_indices(self, ast):
        return ast

    def for_index(self, ast):
        return ast

    def while_statement(self, ast):
        return ast

    def when_equation(self, ast):
        return ast

    def when_statement(self, ast):
        return ast

    def connect_clause(self, ast):
        return ast

    def expression(self, ast):
        return ast

    def simple_expression(self, ast):
        return ast

    def logical_expression(self, ast):
        return ast

    def logical_term(self, ast):
        return ast

    def logical_factor(self, ast):
        return ast

    def relation(self, ast):
        return ast

    def rel_op(self, ast):
        return ast

    def arithmetic_expression(self, ast):
        return ast

    def add_op(self, ast):
        return ast

    def term(self, ast):
        return ast

    def mul_op(self, ast):
        return ast

    def factor(self, ast):
        return ast

    def primary(self, ast):
        return ast

    def name(self, ast):
        return ast

    def component_reference(self, ast):
        return ast

    def function_call_args(self, ast):
        return ast

    def function_arguments(self, ast):
        return ast

    def named_arguments(self, ast):
        return ast

    def named_argument(self, ast):
        return ast

    def function_argument(self, ast):
        return ast

    def output_expression_list(self, ast):
        return ast

    def expression_list(self, ast):
        return ast

    def array_subscripts(self, ast):
        return ast

    def subscript(self, ast):
        return ast

    def comment(self, ast):
        return ast

    def string_comment(self, ast):
        return ast

    def annotation(self, ast):
        return ast

    def algorithm(self, ast):
        return ast

    def and_(self, ast):
        return ast

    def assert_(self, ast):
        return ast

    def block(self, ast):
        return ast

    def break_(self, ast):
        return ast

    def class_(self, ast):
        return ast

    def connect(self, ast):
        return ast

    def connector(self, ast):
        return ast

    def constant(self, ast):
        return ast

    def constrainedby(self, ast):
        return ast

    def der(self, ast):
        return ast

    def discrete(self, ast):
        return ast

    def each(self, ast):
        return ast

    def else_(self, ast):
        return ast

    def elseif(self, ast):
        return ast

    def elsewhen(self, ast):
        return ast

    def encapsulated(self, ast):
        return ast

    def end(self, ast):
        return ast

    def enumeration(self, ast):
        return ast

    def expandable(self, ast):
        return ast

    def extends(self, ast):
        return ast

    def external(self, ast):
        return ast

    def false(self, ast):
        return ast

    def final(self, ast):
        return ast

    def flow(self, ast):
        return ast

    def for_(self, ast):
        return ast

    def function(self, ast):
        return ast

    def if_(self, ast):
        return ast

    def import_(self, ast):
        return ast

    def impure(self, ast):
        return ast

    def in_(self, ast):
        return ast

    def initial(self, ast):
        return ast

    def inner(self, ast):
        return ast

    def input(self, ast):
        return ast

    def loop(self, ast):
        return ast

    def model(self, ast):
        return ast

    def not_(self, ast):
        return ast

    def operator(self, ast):
        return ast

    def or_(self, ast):
        return ast

    def outer(self, ast):
        return ast

    def output(self, ast):
        return ast

    def package(self, ast):
        return ast

    def parameter(self, ast):
        return ast

    def partial(self, ast):
        return ast

    def protected(self, ast):
        return ast

    def public(self, ast):
        return ast

    def pure(self, ast):
        return ast

    def record(self, ast):
        return ast

    def redeclare(self, ast):
        return ast

    def replaceable(self, ast):
        return ast

    def return_(self, ast):
        return ast

    def stream(self, ast):
        return ast

    def then(self, ast):
        return ast

    def true(self, ast):
        return ast

    def type(self, ast):
        return ast

    def when(self, ast):
        return ast

    def while_(self, ast):
        return ast

    def within(self, ast):
        return ast

    def re_IDENT(self, ast):
        return ast

    def re_STRING(self, ast):
        return ast

    def re_UNSIGNED_NUMBER(self, ast):
        return ast

    def IDENT(self, ast):
        return ast

    def STRING(self, ast):
        return ast

    def UNSIGNED_NUMBER(self, ast):
        return ast


def main(filename, startrule, trace=False, whitespace=None):
    import json
    with open(filename) as f:
        text = f.read()
    parser = ModelicaParser(parseinfo=False)
    ast = parser.parse(
        text,
        startrule,
        filename=filename,
        trace=trace,
        whitespace=whitespace)
    print('AST:')
    print(ast)
    print()
    print('JSON:')
    print(json.dumps(ast, indent=2))
    print()

if __name__ == '__main__':
    import argparse
    import string
    import sys

    class ListRules(argparse.Action):
        def __call__(self, parser, namespace, values, option_string):
            print('Rules:')
            for r in ModelicaParser.rule_list():
                print(r)
            print()
            sys.exit(0)

    parser = argparse.ArgumentParser(description="Simple parser for Modelica.")
    parser.add_argument('-l', '--list', action=ListRules, nargs=0,
                        help="list all rules and exit")
    parser.add_argument('-t', '--trace', action='store_true',
                        help="output trace information")
    parser.add_argument('-w', '--whitespace', type=str, default=string.whitespace,
                        help="whitespace specification")
    parser.add_argument('file', metavar="FILE", help="the input file to parse")
    parser.add_argument('startrule', metavar="STARTRULE",
                        help="the start rule for parsing")
    args = parser.parse_args()

    main(args.file, args.startrule, trace=args.trace, whitespace=args.whitespace)



#!/usr/bin/env python
"""A python/modelica based simulation environment.

Pymola contains a Python based compiler for the modelica language
and enables interacting with Modelica easily in Python.

"""
MAJOR = 0
MINOR = 0
MICRO = 3
ISRELEASED = True
VERSION = '%d.%d.%d' % (MAJOR, MINOR, MICRO)

DOCLINES = __doc__.split("\n")

import os
import sys
import subprocess

from setuptools import setup, find_packages

CLASSIFIERS = """\
Development Status :: 1 - Planning
Intended Audience :: Science/Research
Intended Audience :: Developers
License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)
Programming Language :: Python
Programming Language :: Python :: 3
Programming Language :: Other
Topic :: Software Development
Topic :: Scientific/Engineering :: Artificial Intelligence
Topic :: Scientific/Engineering :: Mathematics
Topic :: Scientific/Engineering :: Physics
Topic :: Scientific/Engineering :: Visualization
Operating System :: Microsoft :: Windows
Operating System :: POSIX
Operating System :: Unix
Operating System :: MacOS
Topic :: Software Development :: Code Generators
Topic :: Software Development :: Compilers
Topic :: Software Development :: Embedded Systems
"""


# Return the git revision as a string
def git_version():
    def _minimal_ext_cmd(cmd):
        # construct minimal environment
        env = {}
        for k in ['SYSTEMROOT', 'PATH']:
            v = os.environ.get(k)
            if v is not None:
                env[k] = v
        # LANGUAGE is used on win32
        env['LANGUAGE'] = 'C'
        env['LANG'] = 'C'
        env['LC_ALL'] = 'C'
        out = subprocess.Popen(
            cmd,
            stdout=subprocess.PIPE,
            env=env).communicate()[0]
        return out

    try:
        out = _minimal_ext_cmd(['git', 'rev-parse', 'HEAD'])
        GIT_REVISION = out.strip().decode('ascii')
    except OSError:
        GIT_REVISION = "Unknown"

    return GIT_REVISION


def get_version_info():
    # Adding the git rev number needs to be done inside write_version_py(),
    # otherwise the import of package.version messes up
    # the build under Python 3.
    FULLVERSION = VERSION
    if os.path.exists('.git'):
        GIT_REVISION = git_version()
    elif os.path.exists('pymola/version.py'):
        # must be a source distribution, use existing version file
        try:
            from pymola.version import git_revision as GIT_REVISION
        except ImportError:
            raise ImportError("Unable to import git_revision. Try removing "
                              "pymola/version.py and the build directory "
                              "before building.")
    else:
        GIT_REVISION = "Unknown"

    if not ISRELEASED:
        FULLVERSION += '.dev-' + GIT_REVISION[:7]

    return FULLVERSION, GIT_REVISION


def write_version_py(filename='pymola/version.py'):
    cnt = """
# THIS FILE IS GENERATED FROM SETUP.PY
short_version = '%(version)s'
version = '%(version)s'
full_version = '%(full_version)s'
git_revision = '%(git_revision)s'
release = %(isrelease)s

if not release:
    version = full_version
"""
    FULLVERSION, GIT_REVISION = get_version_info()

    a = open(filename, 'w')
    try:
        a.write(cnt % {'version': VERSION,
                       'full_version': FULLVERSION,
                       'git_revision': GIT_REVISION,
                       'isrelease': str(ISRELEASED)})
    finally:
        a.close()


def setup_package():
    src_path = os.path.dirname(os.path.abspath(sys.argv[0]))
    old_path = os.getcwd()
    os.chdir(src_path)
    sys.path.insert(0, src_path)

    # Rewrite the version file everytime
    write_version_py()

    metadata = dict(
        name='pymola',
        maintainer="James Goppert",
        maintainer_email="james.goppert@gmail.com",
        description=DOCLINES[0],
        long_description="\n".join(DOCLINES[2:]),
        url='https://github.com/jgoppert/pymola',
        author='James Goppert',
        author_email='james.goppert@gmail.com',
        download_url='https://github.com/jgoppert/pymola',
        license='GPLv3+',
        classifiers=[_f for _f in CLASSIFIERS.split('\n') if _f],
        platforms=["Windows", "Linux", "Solaris", "Mac OS-X", "Unix"],
        install_requires=['parsimonious', 'ply', 'grako'],
        tests_require=['nose'],
        test_suite='nose.collector',
        packages=find_packages(exclude=['*.test']),
    )

    print('packages: ', metadata['packages'])
    FULLVERSION, GIT_REVISION = get_version_info()
    metadata['version'] = FULLVERSION

    try:
        setup(**metadata)
    finally:
        del sys.path[0]
        os.chdir(old_path)
    return

if __name__ == '__main__':
    setup_package()
